import { _decorator, Animation, Component, EventTouch, instantiate, Node, ParticleSystem2D, Prefab, Vec2, Vec3 } from 'cc';
const { ccclass, property } = _decorator;

@ccclass('Main')
export class Main extends Component {
    
    @property(Prefab)
    public effectPrefab: Prefab;

    private posV2: Vec2 = new Vec2();
    private posV3: Vec3 = new Vec3();

    private _pool: Array<Node> = [];

    protected onLoad(): void {
        this.node.on(Node.EventType.TOUCH_START, this.showTouchEffect, this, true);
    }

    private showTouchEffect(eventTouch: EventTouch) {
        eventTouch.getUILocation(this.posV2);
        this.posV3.x = this.posV2.x;
        this.posV3.y = this.posV2.y;

        let effectNode = this.getEffectNode();
        this.node.addChild(effectNode);
        effectNode.setWorldPosition(this.posV3);

        
        let anima = effectNode.getComponentInChildren(Animation);
        let callback = () => {
            anima.off(Animation.EventType.FINISHED, callback, this);
            effectNode.removeFromParent();
            this._pool.push(effectNode);
        }
        anima.on(Animation.EventType.FINISHED, callback, this);
        anima.play();

        let pars = effectNode.getComponentsInChildren(ParticleSystem2D);
        for (let particle of pars) {
            particle.resetSystem();
        }

    }

    private getEffectNode(): Node {
        if (this._pool.length > 0) {
            return this._pool.pop();
        }

        let node = instantiate(this.effectPrefab);
        return node;
    }
}